/*****************************************************************************
 * 
 * mui-gtk - a wrapper library to wrap GTK+ calls to MUI
 *
 * Copyright (C) 2005 Oliver Brunner
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 * Contact information:
 *
 * Oliver Brunner
 *
 * E-Mail: mui-gtk "at" oliver-brunner.de
 *
 * $Id$
 *
 *****************************************************************************/

#include <stdlib.h>
#include <proto/utility.h>
#include <mui.h>
/*
#include <clib/alib_protos.h>
#include <exec/types.h>
*/
#include "debug.h"
#include "classes.h"
#include "gtk.h"
#include "gtk_globals.h"

#ifndef __AROS__
  #define MGTK_HOOK_OBJECT_POINTER Object *
  #define MGTK_HOOK_APTR APTR
  #define MGTK_USERFUNC_INIT
  #define MGTK_USERFUNC_EXIT
  #include <SDI_hook.h>
#else
  #include "mgtk_aros.h"
#endif

struct Data
{
  GtkWidget *widget;
  Object *box;
  Object *radio;
  char *label;
};

static GtkWidget *mgtk_get_widget_from_obj(Object *obj) {
  Object *o;

  o=obj;
  while(!xget(o,MA_Widget)) {
    DebOut("  o=%x\n",o);
    o=_parent(o);
  }
  return (GtkWidget *) xget(o,MA_Widget);
}

/*******************************************
 * mgtk_radio_value_update(GtkWidget *widget)
 *
 *******************************************/
 /*internal */
static void mgtk_radio_value_update0(GtkWidget *widget,struct Data *data, int b) {

  DebOut("mgtk_radio_value_update0(%x,%x,%d)\n",widget,data,b);

  DebOut("  data->radio(%lx)=%d\n",data->radio,b);

  nnset(data->radio,MUIA_Selected,b);
}

/*******************************************
 * MUIHook_radiobutton
 *
 * clear/set all other widget objects in
 * our group and emit the right signal(s)
 *******************************************/
HOOKPROTO(MUIHook_radio, ULONG, MGTK_HOOK_OBJECT_POINTER obj, MGTK_HOOK_APTR hookpointer)
{
  MGTK_USERFUNC_INIT
  GtkWidget *active;
  GtkWidget *act;
  GSList *next;

  DebOut("MUIHook_radio called\n");
  active=mgtk_get_widget_from_obj(obj);
  DebOut("  widget=%x\n",(int) active);

  if(!active->active) {
    next=gtk_radio_button_get_group(active);
    DebOut("  group length: %ld\n",g_slist_length(next));

    while(next!=NULL) {
      act=next->data;
      /* clear the previous active object */
      if(act->active) {
        nnset(act->MuiObject,MA_Radio_Value,0);
        act->active=0;
      }
      next=g_slist_next(next);
    }
    /* make our object the active one */
    active->active=1;
  }
  nnset(active->MuiObject,MA_Radio_Value,1);

  DebOut("  emit newstyle toggled signal\n");
  g_signal_emit(active,g_signal_lookup("toggled",0),0);

  return 0;
  MGTK_USERFUNC_EXIT
}
MakeHook(MyMuiHook_radio, MUIHook_radio);

/*******************************************
 * mNew
 * 
 * requires MA_Widget!
 *******************************************/
static ULONG mNew(struct IClass *cl, APTR obj, Msg msg) {
  Object *box;
  Object *radio;
  GtkWidget *widget;
  const char *label;
  char *mylabel;
  struct TagItem *tstate, *tag;

  DebOut("mNew (radiobutton)\n");

  obj = (APTR)DoSuperMethodA(cl, obj, msg);

  if(!obj) {
    printf("ERROR: radiobutton: unable to create object!");
    return (ULONG) NULL;
  }

  DebOut("  new obj=%lx\n",obj);

  tstate=((struct opSet *)msg)->ops_AttrList;

  widget=NULL;
  label=NULL;
  while ((tag = (struct TagItem *) NextTagItem(&tstate)))
  {
    switch (tag->ti_Tag)
    {
      case MA_Widget:
        widget = (GtkWidget *) tag->ti_Data;
        break;
      case MA_Radio_Label:
        label = (const char *) tag->ti_Data;
        break;
    }
  }

  if(!widget) {
    printf("ERROR: classes/radiobutton.c: mNew: MA_Widget not supplied!\n");
    return (ULONG) NULL;
  }

  DebOut("  widget=%d\n",widget);

  if(label) {
    mylabel=g_strdup(label);
  }
  else {
    mylabel=g_strdup("");
  }

  box=(APTR) HGroup, 
          MUIA_CycleChain, 1,
          Child,
            radio = (APTR) ImageObject,
            NoFrame,
            MUIA_InputMode        , MUIV_InputMode_Toggle,
            MUIA_Image_Spec       , MUII_RadioButton,
            MUIA_ShowSelState     , FALSE,
          End,
          Child, Label1(mylabel),
          Child, HSpace(0),
        End;

  DoMethod(obj,OM_ADDMEMBER,box);

  /* setup internal hooks */
  DoMethod(radio,MUIM_Notify,MUIA_Selected,MUIV_EveryTime, radio,2,MUIM_CallHook,&MyMuiHook_radio);

  GETDATA;

  data->box=box;
  data->radio=radio;
  data->label=mylabel;
  data->widget=widget;

  return (ULONG)obj;
}

/*******************************************
 * mSet
 * 
 * MA_Radio_Value only usefull TAG here
 * MA_Radio_String_Value *private*
 *******************************************/
static VOID mSet(struct Data *data, APTR obj, struct opSet *msg) {
	DebOut("mSet(%lx,%lx,%lx)\n",data,obj,msg);

  struct TagItem *tstate, *tag;
  GtkWidget *widget;

  tstate = msg->ops_AttrList;

  while ((tag = (struct TagItem *) NextTagItem(&tstate)))
  {
    switch (tag->ti_Tag)
    {
      case MA_Radio_Value:
        DebOut("mSet(%lx) to %d\n",obj,tag->ti_Data);
        widget=mgtk_get_widget_from_obj(obj);
        mgtk_radio_value_update0(widget,data,(int)tag->ti_Data);
        break;
    }
  }
}

/*******************************************
 * mGet
 * 
 * MA_Radio_Value: value as string
 * MA_Widget: GtkWidget of this radio
 *******************************************/
static ULONG mGet(struct Data *data, APTR obj, struct opGet *msg, struct IClass *cl) {
  ULONG rc;

  switch (msg->opg_AttrID) {
    case MA_Widget : 
      DebOut("mGet: data->widget=%x\n",(int) data->widget);
      rc = (ULONG) data->widget;
      break;
    case MA_Radio_Value : 
      rc=xget(data->radio, MUIA_Selected);
      DebOut("mGet: MA_Radio_Value =%d\n", rc);
      break;
    default: return DoSuperMethodA(cl, obj, (Msg)msg);
  }

  *msg->opg_Storage = rc;
  return TRUE;
}

/*******************************************
 * Dispatcher
 *******************************************/
BEGINMTABLE
GETDATA;

  switch (msg->MethodID)
  {
    case OM_NEW         : return mNew       (cl, obj, msg);
    case OM_SET         :        mSet       (data, obj, (APTR)msg); break;
    case OM_GET         : return mGet       (data, obj, (APTR)msg, cl);
  }

ENDMTABLE

/*******************************************
 * Custom class create/delete
 *******************************************/
int mgtk_create_radiobutton_class(void)
{
  DebOut("mgtk_create_radiobutton_class()\n");

  CL_RadioButton = MUI_CreateCustomClass(NULL, MUIC_Group, NULL, sizeof(struct Data), (APTR)&mDispatcher);

  DebOut("CL_RadioButton=%lx\n",CL_RadioButton);

  return CL_RadioButton ? 1 : 0;
}

void mgtk_delete_radiobutton_class(void)
{
  if (CL_RadioButton)
  {
    MUI_DeleteCustomClass(CL_RadioButton);
  }
}
