/*****************************************************************************
 * 
 * mui-gtk - a wrapper library to wrap GTK+ calls to MUI
 *
 * Copyright (C) 2005 Oliver Brunner
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 * Contact information:
 *
 * Oliver Brunner
 *
 * E-Mail: mui-gtk "at" oliver-brunner.de
 *
 * $Id: gtk_window.c,v 1.10 2005/08/05 12:29:47 o1i Exp $
 *
 *****************************************************************************/

#include <stdio.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#include <proto/exec.h>
#include <mui.h>

/* cross platform hooks */
#include <SDI_hook.h>

#include "classes.h"
#include "debug.h"
#include "gtk.h"
#include "gtk_globals.h"

/*********************************
 * gtk_window_new
 *********************************/
GtkWidget* gtk_window_new   (GtkWindowType   type) {

  GtkWidget *win;
  GSList *gl;

  DebOut("gtk_window_new: TODO: check parameter\n");

  win=mgtk_widget_new(IS_WINDOW);

  gl=mgtk->windows;

  if(!g_slist_length(gl)) {
    /* first window */
    DebOut("  first window\n");

    win->MuiWin=mgtk->MuiRoot;
    win->MuiWinMenustrip=mgtk->MuiRootStrip;
    win->MuiGroup=mgtk->Group;
    win->MuiObject=mgtk->Group;
    /* mgtk->winnr++; */
    gl=g_slist_append(gl,win);
    mgtk->windows=gl;
    return win;
  }

  printf("WARNING: gtk_window_new: second window not done yet\n!");
  return(NULL);
}

void gtk_window_set_title   (GtkWindow *window, const gchar *title) {
  DebOut("gtk_window_set_title(win %lx,%s)\n",window,title);

  SetAttrs(window->MuiWin,MUIA_Window_Title,(ULONG) title, TAG_DONE); /* ouch, casting it to ULONG.. */
}

void gtk_window_set_resizable(GtkWindow *window, gboolean resizable) {
#if 0
  ULONG is_open, was_resizable;
#endif

  DebOut("gtk_window_set_resizable(%lx,%d)\n",window,resizable);

#if 0
  was_resizable = xget(window->MuiWin, MUIA_Window_SizeGadget);
  is_open = xget(window->MuiWin, MUIA_Window_Open);

  SetAttrs(window->MuiWin,MUIA_Window_SizeGadget,(ULONG) resizable, TAG_DONE);

  if (is_open && !(was_resizable & resizable))
  {
    /* We must reopen window to toggle sizing gadget
    **
    ** This causes unavoidable flickery. It could be avoided by rootgroup subclass
    ** (by making group fixed size) but the sizing gadget remains.
    **
    ** Hmm...
    */

    SetAttrs(window->MuiWin, MUIA_Window_Open, FALSE, TAG_DONE);
    SetAttrs(window->MuiWin, MUIA_Window_Open, TRUE, TAG_DONE);
  }
#else
	/* Drawback in this method: sizing gadget is still there
	**
	** OTOH is there any good reason to disable resizing? Because GTK coders are lame?
  **
  ** Disable this
	*/

  set(window->MuiObject, MA_RootGroup_Resizable, resizable);
#endif

  return;
}

gboolean gtk_window_get_resizable(GtkWindow *window)
{
	return xget(window->MuiWin, MUIA_Window_SizeGadget);
}
